import { Switch, Route } from "wouter";
import { QueryClientProvider } from "@tanstack/react-query";
import { queryClient, fetchCsrfToken } from "./lib/queryClient";
import { ToastContainer } from "react-toastify";
import { TooltipProvider } from "@/components/ui/tooltip";
import { HelmetProvider } from "react-helmet-async";
// Import the react-toastify CSS and our custom styles
import "react-toastify/dist/ReactToastify.css";
import "@/styles/toast.css";
import "./styles/tour-guide.css";
import { AuthProvider } from "@/hooks/use-auth";
import { CartProvider } from "@/hooks/use-cart";
import { OfflineProvider } from "@/hooks/use-offline";
import { CookieConsentProvider } from "@/hooks/use-cookie-consent";
import { OfflineBanner } from "@/components/OfflineBanner";
import { SlowConnectionBanner } from "@/components/SlowConnectionBanner";
import { CookieBanner } from "@/components/CookieBanner";
import { UpdateNotification } from "@/components/UpdateNotification";
import ProDevToolbar from "@/components/ProDevToolbar";
import { UIStateProvider } from "@/context/UIStateContext";
import { SiteModeProvider, useSiteMode } from "@/context/SiteModeContext";
import Navbar from "./layout/Navbar";
import Footer from "@/components/Footer";
import { lazy, Suspense, memo, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { TranslationLoader } from "@/components/TranslationLoader";
import { AnimatedLoader } from "@/components/ui/animated-loader";

// Implement code splitting with lazy loading for all pages
const Home = lazy(() => import("./pages/Home"));
const AuthPage = lazy(() => import("./pages/auth-page"));
const AccountPage = lazy(() => import("./pages/account-page"));
const OnboardingPage = lazy(() => import("./pages/onboarding"));
const VehicleLookup = lazy(() => import("./pages/VehicleLookup"));
const TireSelection = lazy(() => import("./pages/TireSelection"));
const ServiceSelection = lazy(() => import("./pages/ServiceSelection"));
const FittingPreference = lazy(() => import("./pages/FittingPreference"));
const Checkout = lazy(() => import("./pages/Checkout"));
const OrderTracking = lazy(() => import("./pages/OrderTracking"));
const VehicleHealthReportPage = lazy(() => import("./pages/VehicleHealthReportPage"));
const SmartSearchPage = lazy(() => import("./pages/SmartSearchPage"));
const ComingSoon = lazy(() => import("./pages/ComingSoon"));
const MaintenancePage = lazy(() => import("./pages/MaintenancePage"));

const NotFound = lazy(() => import("./pages/not-found"));

// Import all our detailed page-specific skeleton loaders
import {
  HeroSkeleton,
  SearchFormSkeleton,
  FeatureGridSkeleton,
  VehicleLookupSkeleton,
  VehicleDetailSkeleton,
  TireSelectionSkeleton,
  SectionSkeleton,
  AccountPageSkeleton,
  CheckoutSkeleton,
  HomePageSkeleton,
  AuthPageSkeleton,
  FittingFormSkeleton
} from "@/components/ui/skeleton-loaders";

// Dynamic loading component that appears during code splitting
// Shows different skeleton layouts or AnimatedLoader based on the current route
const PageLoader = () => {
  // Use the enhanced AnimatedLoader for all routes
  return <AnimatedLoader />;
  
  // The following code is disabled to ensure consistent loading experience
  /*
  // Check URL to determine which page skeleton to show
  const path = window.location.pathname;
  
  // Homepage skeleton - uses our dedicated comprehensive HomePageSkeleton component
  if (path === "/") {
    return <HomePageSkeleton className="p-8" />;
  }
  
  // Account page skeleton
  if (path.includes("account")) {
    return <AccountPageSkeleton className="p-8" />;
  }
  
  // Vehicle detail page skeleton
  if (path.includes("vehicle")) {
    return <VehicleDetailSkeleton className="p-8" />;
  }
  */
  
  // All conditional skeletal loaders have been disabled in favor of using the AnimatedLoader consistently
  // This ensures a seamless loading experience across the app and prevents any potential car spinner visuals
};

// Standalone page router without Navbar and Footer
const StandaloneRouter = memo(() => {
  // Get the current path and site mode from the context
  const [path, setPath] = useState(window.location.pathname);
  const { mode: contextMode } = useSiteMode();
  
  // Keep path in sync with window.location (for direct navigation)
  useEffect(() => {
    const updatePath = () => {
      setPath(window.location.pathname);
    };
    
    window.addEventListener('popstate', updatePath);
    return () => window.removeEventListener('popstate', updatePath);
  }, []);
  
  // Merge context mode with environment variable as fallback
  const getSiteMode = () => {
    // First try the context (which should be from API/server)
    if (contextMode) {
      console.log('🌐 StandaloneRouter using mode from context:', contextMode);
      return contextMode;
    }
    
    // Then try the environment variable
    const envMode = import.meta.env.VITE_SITE_MODE;
    if (envMode) {
      console.log('🌐 StandaloneRouter using mode from env:', envMode);
      return envMode;
    }
    
    // Last resort - check localStorage
    try {
      const localMode = localStorage.getItem('PITSTOP360_SITE_MODE') || localStorage.getItem('SITE_MODE');
      if (localMode) {
        console.log('🌐 StandaloneRouter using mode from localStorage:', localMode);
        return localMode;
      }
    } catch (err) {
      console.error('🌐 StandaloneRouter error reading from localStorage:', err);
    }
    
    // Default fallback
    return 'normal';
  };
  
  const currentMode = getSiteMode();
  
  // Update the URL to match the current mode
  useEffect(() => {
    // Determine target path based on mode
    let targetPath = path;
    
    if (currentMode === 'coming-soon' && path !== '/coming-soon') {
      targetPath = '/coming-soon';
    } else if (currentMode === 'maintenance' && path !== '/maintenance') {
      targetPath = '/maintenance';
    }
    
    // Only update if we're changing paths
    if (targetPath !== path) {
      console.log(`🌐 StandaloneRouter updating URL from ${path} to ${targetPath}`);
      window.history.pushState({}, '', targetPath);
      setPath(targetPath);
    }
  }, [path, currentMode]);
  
  console.log(`🌐 StandaloneRouter rendering, path: ${path}, mode: ${currentMode}`);
  
  // Simplified component rendering based on path and mode
  const getComponentToRender = () => {
    if (path === '/coming-soon' || currentMode === 'coming-soon') {
      return <ComingSoon />;
    } else if (path === '/maintenance' || currentMode === 'maintenance') {
      return <MaintenancePage />;
    }
    
    // Fallback (shouldn't normally happen)
    console.warn('🌐 StandaloneRouter: No matching path or mode, using ComingSoon as fallback');
    return <ComingSoon />;
  };
  
  return (
    <div className="min-h-screen bg-background">
      <Suspense fallback={<PageLoader />}>
        {getComponentToRender()}
      </Suspense>
    </div>
  );
});

// Main application router with Navbar and Footer
const MainRouter = memo(() => {
  return (
    <div className="min-h-screen bg-background">
      <UIStateProvider>
        <Navbar />
        <OfflineBanner />
        <SlowConnectionBanner />
        <main className="min-h-[calc(100vh-4rem)]">
          <Suspense fallback={<PageLoader />}>
            <Switch>
              <Route path="/" component={Home} />
              <Route path="/auth" component={AuthPage} />
              <Route path="/account" component={AccountPage} />
              {/* Onboarding page removed - users go straight to account */}
              <Route path="/vehicle-lookup" component={VehicleLookup} />
              <Route path="/tire-selection" component={TireSelection} />
              <Route path="/service-selection" component={ServiceSelection} />
              <Route path="/fitting-preference" component={FittingPreference} />
              <Route path="/checkout" component={Checkout} />
              <Route path="/order/:id" component={OrderTracking} />
              <Route path="/vehicle-health-report/:id?" component={VehicleHealthReportPage} />
              <Route path="/smart-search" component={SmartSearchPage} />
              
              <Route component={NotFound} />
            </Switch>
          </Suspense>
        </main>
        <Footer />
      </UIStateProvider>
    </div>
  );
});

// Combined router that checks if we should use the standalone router or the main router
const Router = memo(() => {
  // Get the current path and site mode
  const path = window.location.pathname;
  const { mode } = useSiteMode();
  
  // Check from context first, but also check localStorage as a fallback
  const checkSiteMode = () => {
    // First use the context mode which should be properly synced with the API/server
    if (mode) {
      console.log('🌐 Router using site mode from context:', mode);
      return mode;
    }
    
    // If context isn't ready, check environment variable (added redundancy)
    const envMode = import.meta.env.VITE_SITE_MODE;
    if (envMode) {
      console.log('🌐 Router using site mode from env:', envMode);
      return envMode;
    }
    
    try {
      // Finally, check localStorage as last resort
      const siteMode = localStorage.getItem('PITSTOP360_SITE_MODE') || localStorage.getItem('SITE_MODE') || 'normal';
      console.log('🌐 Router using site mode from localStorage:', siteMode);
      return siteMode;
    } catch (err) {
      console.error('Error reading site mode from localStorage:', err);
      return 'normal';
    }
  };
  
  const currentSiteMode = checkSiteMode();
  
  // If we're in a restricted mode (coming-soon or maintenance) or accessing a restricted path,
  // show the standalone router - this ensures a consistent user experience
  if (currentSiteMode === 'coming-soon' || 
      currentSiteMode === 'maintenance' || 
      path === '/coming-soon' || 
      path === '/maintenance') {
    console.log(`🌐 Router showing standalone page (mode: ${currentSiteMode}, path: ${path})`);
    return <StandaloneRouter />;
  }
  
  // Otherwise, use the main router
  console.log('🌐 Router showing main application');
  return <MainRouter />;
});

// Loading component for translations removed as it was creating duplicate loading screens
// We now use the AnimatedLoader component exclusively

// Component that fetches CSRF token and monitors i18n state (without re-initializing it)
function AppInitializer() {
  // Access i18n instance without triggering re-initialization
  const { i18n } = useTranslation();
  
  // Fetch CSRF token at startup
  useEffect(() => {
    // Fetch the CSRF token on app startup
    fetchCsrfToken().catch((error) => {
      console.warn('Failed to fetch initial CSRF token:', error);
      // We'll retry on first API request, so no need to show an error
    });
  }, []);
  
  // Only log i18n state on mount, not on every render
  useEffect(() => {
    // Log detailed information about the i18n state (but only once)
    console.log('[AppInitializer] i18n state check:');
    console.log('→ i18n is initialized:', i18n?.isInitialized);
    console.log('→ Supported languages:', i18n?.languages);
    console.log('→ Current language:', i18n?.language);
    
    // Handle potentially undefined i18n options
    const namespaces = i18n?.options?.ns || ['translation'];
    console.log('→ Loaded namespaces:', namespaces);
    
    // Check if localStorage flag exists but i18n is not actually initialized
    try {
      const storedFlag = localStorage.getItem('__PitStop360_i18n_initialized__');
      if (storedFlag === 'true' && !i18n?.isInitialized) {
        console.warn('[AppInitializer] LocalStorage shows i18n as initialized, but i18n instance is not ready. Clearing flag for re-initialization.');
        localStorage.removeItem('__PitStop360_i18n_initialized__');
        // Force a page reload to reinitialize i18n properly
        window.location.reload();
      }
    } catch (err) {
      console.error('[AppInitializer] Error checking i18n state:', err);
    }
  }, [i18n]);
  
  return null;
}

// Non-critical providers that can be lazy loaded
const LazyProviders = lazy(() => 
  Promise.all([
    import("@/hooks/use-cart")
  ]).then(([cartModule]) => ({
    default: ({ children }: { children: React.ReactNode }) => (
      <cartModule.CartProvider>
        {children}
      </cartModule.CartProvider>
    )
  }))
);

// Critical providers needed for initial render
function CriticalProviders({ children }: { children: React.ReactNode }) {
  // Make sure to use simple provider wrapping structure that's consistent
  return (
    <QueryClientProvider client={queryClient}>
      <TooltipProvider>
        <AuthProvider>
          <OfflineProvider>
            <CookieConsentProvider>
              {children}
            </CookieConsentProvider>
          </OfflineProvider>
        </AuthProvider>
      </TooltipProvider>
    </QueryClientProvider>
  );
}

function App() {
  return (
    <HelmetProvider>
      <CriticalProviders>
        <AppInitializer />
        {/* Use a single AnimatedLoader for all initial loading */}
        <Suspense fallback={<AnimatedLoader />}>
          <TranslationLoader>
            <SiteModeProvider>
              <LazyProviders>
                <Router />
                <CookieBanner />
                <UpdateNotification />
                <ToastContainer />
                {/* Keep the main ProDevToolbar as requested */}
                {process.env.NODE_ENV !== 'production' && <ProDevToolbar />}
              </LazyProviders>
            </SiteModeProvider>
          </TranslationLoader>
        </Suspense>
      </CriticalProviders>
    </HelmetProvider>
  );
}

export default App;
